/*
 * Jalview - A Sequence Alignment Editor and Viewer (Version 2.4)
 * Copyright (C) 2008 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.appletgui;

import java.util.*;

import java.awt.*;
import java.awt.event.*;

import jalview.datamodel.*;
import jalview.schemes.*;
import java.awt.Rectangle;

public class FeatureColourChooser extends Panel implements
        ActionListener, AdjustmentListener, ItemListener, MouseListener
{
  Frame frame;

  FeatureRenderer fr;
  FeatureSettings fs;
  AlignmentPanel ap;

  GraduatedColor cs;
  Object oldcs;

  Hashtable oldgroupColours;


  boolean adjusting = false;
  private float min,max;
  String type=null;
  
  public FeatureColourChooser(FeatureSettings fsettings, String type)
  {
    this.fs = fsettings;
    this.type = type;
    fr = fsettings.fr;
    ap = fsettings.ap;
    float mm[] = ((float[][]) fr.minmax.get(type))[0];
    min = mm[0];
    max = mm[1];
    oldcs = fr.featureColours.get(type);
    if (oldcs instanceof GraduatedColor)
    {
      cs = new GraduatedColor((GraduatedColor) oldcs, min, max);
    } else {
      // promote original color to a graduated color
      Color bl = Color.black;
      if (oldcs instanceof Color)
      {
        bl = (Color) oldcs;
      }
      // original colour becomes the maximum colour
      cs = new GraduatedColor(Color.white,bl,mm[0],mm[1]);
    }
    minColour.setBackground(cs.getMinColor());
    maxColour.setBackground(cs.getMaxColor());
    adjusting = true;

    try
    {
      jbInit();
    } catch (Exception ex)
    {
    }
    // To HERE!
    adjusting = false;
    changeColour();
    slider.addAdjustmentListener(this);
    slider.addMouseListener(this);
    frame = new Frame();
    frame.add(this);
    jalview.bin.JalviewLite.addFrame(frame, "Graduated Feature Colour for "+type, 480,
            145);
    validate();
  }

  public FeatureColourChooser()
  {
    try
    {
      jbInit();
    } catch (Exception ex)
    {
      ex.printStackTrace();
    }
  }

  private void jbInit() throws Exception
  {
    minColour.setFont(new java.awt.Font("Verdana", Font.PLAIN, 11));
    minColour.setLabel("Min Colour");
    minColour.addActionListener(this);

    maxColour.setFont(new java.awt.Font("Verdana", Font.PLAIN, 11));
    maxColour.setLabel("Max Colour");
    maxColour.addActionListener(this);

    thresholdIsMin.addItemListener(this);
    ok.setLabel("OK");
    ok.addActionListener(this);

    cancel.setLabel("Cancel");
    cancel.addActionListener(this);

    this.setLayout(borderLayout1);
    jPanel2.setLayout(flowLayout1);

    jPanel1.setBackground(Color.white);
    jPanel2.setBackground(Color.white);
    threshold.addItemListener(this);
    threshold.addItem("No Threshold");
    threshold.addItem("Above Threshold");
    threshold.addItem("Below Threshold");
    jPanel3.setLayout(null);
    thresholdValue.addActionListener(this);

    slider.setBackground(Color.white);
    slider.setEnabled(false);
    slider.setBounds(new Rectangle(153, 3, 93, 21));
    thresholdValue.setEnabled(false);
    thresholdValue.setBounds(new Rectangle(248, 2, 79, 22));
    thresholdValue.setColumns(5);
    jPanel3.setBackground(Color.white);
    //currentColours.setFont(new java.awt.Font("Verdana", Font.PLAIN, 11));
    //currentColours.setLabel("Use Original Colours");
    //currentColours.addItemListener(this);

    threshold.setBounds(new Rectangle(11, 3, 139, 22));
    thresholdIsMin.setBackground(Color.white);
    thresholdIsMin.setLabel("Threshold is min/max");
    thresholdIsMin.setBounds(new Rectangle(328, 3, 135, 23));
    jPanel1.add(ok);
    jPanel1.add(cancel);
    //jPanel2.add(currentColours);
    jPanel2.add(minColour);
    jPanel2.add(maxColour);
    jPanel3.add(threshold);
    jPanel3.add(slider);
    jPanel3.add(thresholdValue);
    jPanel3.add(thresholdIsMin);
    this.add(jPanel2, java.awt.BorderLayout.NORTH);
    this.add(jPanel3, java.awt.BorderLayout.CENTER);
    this.add(jPanel1, java.awt.BorderLayout.SOUTH);
  }

  Button minColour = new Button();

  Button maxColour = new Button();

  Button ok = new Button();

  Button cancel = new Button();

  Panel jPanel1 = new Panel();

  Panel jPanel2 = new Panel();

  Choice threshold = new Choice();

  FlowLayout flowLayout1 = new FlowLayout();

  Panel jPanel3 = new Panel();

  Scrollbar slider = new Scrollbar(Scrollbar.HORIZONTAL);

  TextField thresholdValue = new TextField(20);


  BorderLayout borderLayout1 = new BorderLayout();

  Checkbox thresholdIsMin = new Checkbox();

  private GraphLine threshline;

  public void actionPerformed(ActionEvent evt)
  {
    if (evt.getSource() == thresholdValue)
    {
      try
      {
        float f = new Float(thresholdValue.getText()).floatValue();
        slider.setValue((int) (f * 1000));
        adjustmentValueChanged(null);
      } catch (NumberFormatException ex)
      {
      }
    }
    else if (evt.getSource() == minColour)
    {
      minColour_actionPerformed(null);
    }
    else if (evt.getSource() == maxColour)
    {
      maxColour_actionPerformed(null);
    }

    else if (evt.getSource() == ok)
    {
      changeColour();
      frame.setVisible(false);
    }
    else if (evt.getSource() == cancel)
    {
      reset();
      ap.paintAlignment(true);
      frame.setVisible(false);
    }

    else
    {
      changeColour();
    }
  }

  public void itemStateChanged(ItemEvent evt)
  {

    changeColour();
  }

  public void adjustmentValueChanged(AdjustmentEvent evt)
  {
    if (!adjusting)
    {
      thresholdValue.setText(((float) slider.getValue() / 1000f) + "");
      valueChanged();
    }
  }
  protected void valueChanged() {
    changeColour();
    threshline.value = (float) slider.getValue() / 1000f;
    ap.paintAlignment(false);
  }
  public void minColour_actionPerformed(Color newCol)
  {
    if (newCol != null)
    {
      minColour.setBackground(newCol);
      minColour.repaint();
      changeColour();
    }
    else
    {
      new UserDefinedColours(this, "Select Colour for Minimum Value", minColour.getBackground());
    }

  }

  public void maxColour_actionPerformed(Color newCol)
  {
    if (newCol != null)
    {
      maxColour.setBackground(newCol);
      maxColour.repaint();
      changeColour();
    }
    else
    {
      new UserDefinedColours(this, "Select Colour for Maximum Value", maxColour.getBackground());
    }
  }

  void changeColour()
  {
    // Check if combobox is still adjusting
    if (adjusting)
    {
      return;
    }

    int aboveThreshold = AnnotationColourGradient.NO_THRESHOLD;
    if (threshold.getSelectedItem().equals("Above Threshold"))
    {
      aboveThreshold = AnnotationColourGradient.ABOVE_THRESHOLD;
    }
    else if (threshold.getSelectedItem().equals("Below Threshold"))
    {
      aboveThreshold = AnnotationColourGradient.BELOW_THRESHOLD;
    }

    slider.setEnabled(true);
    thresholdValue.setEnabled(true);
    GraduatedColor acg = new GraduatedColor(minColour.getBackground(), maxColour.getBackground(), min, max);

    if (aboveThreshold == AnnotationColourGradient.NO_THRESHOLD)
    {
      slider.setEnabled(false);
      thresholdValue.setEnabled(false);
      thresholdValue.setText("");
    }
    
    else if (aboveThreshold != AnnotationColourGradient.NO_THRESHOLD
            && threshline == null)
    {
      // todo visual indication of feature threshold
      threshline = new jalview.datamodel.GraphLine(
                      (max - min) / 2f,
                      "Threshold", Color.black);
    }

    if (aboveThreshold != AnnotationColourGradient.NO_THRESHOLD)
    {
      adjusting = true;
      acg.setThresh(threshline.value);

      float range = max * 1000f
              - min * 1000f;

      slider.setMinimum((int) (min * 1000));
      slider.setMaximum((int) (max * 1000));
      slider.setValue((int) (threshline.value * 1000));
      thresholdValue.setText(threshline.value + "");
      slider.setEnabled(true);
      thresholdValue.setEnabled(true);
      adjusting = false;
    }

    acg.setThreshType(aboveThreshold);
    if (thresholdIsMin.getState() && aboveThreshold != AnnotationColourGradient.NO_THRESHOLD)
    {
      if (aboveThreshold==AnnotationColourGradient.ABOVE_THRESHOLD)
      { 
        acg = new GraduatedColor(acg, threshline.value, max);
      } else { 
        acg = new GraduatedColor(acg, min,threshline.value);
      }
    }
    
    fr.featureColours.put(type,acg);
    cs = acg;
    ap.paintAlignment(false);
  }

  void reset()
  {
    fr.featureColours.put(type, oldcs);
    ap.paintAlignment(true);

  }

  public void mouseClicked(MouseEvent evt)
  {
  }

  public void mousePressed(MouseEvent evt)
  {
  }

  public void mouseReleased(MouseEvent evt)
  {
    ap.paintAlignment(true);
  }

  public void mouseEntered(MouseEvent evt)
  {
  }

  public void mouseExited(MouseEvent evt)
  {
  }

}
